@@ -5,4 +5,4 @@ if [ -f /run/console-setup/keymap_loaded ]; then
     exit 0
 fi
 kbd_mode '-u' 
-loadkeys '/run/tmpkbd.2H0ANu' > '/dev/null' 
+loadkeys '/etc/console-setup/cached_UTF-8_del.kmap.gz' > '/dev/null' 
