@@ -6,26 +6,35 @@ Tests for L{twisted.internet.base}.
 """
 
 import socket
-try:
-    from Queue import Queue
-except ImportError:
-    from queue import Queue
+from queue import Queue
+from typing import Callable
+from unittest import skipIf
 
 from zope.interface import implementer
 
-from twisted.python.threadpool import ThreadPool
-from twisted.internet.interfaces import (IReactorTime, IReactorThreads,
-                                         IResolverSimple)
-from twisted.internet.error import DNSLookupError
+from typing import ParamSpec
+
 from twisted.internet._resolver import FirstOneWins
+from twisted.internet.base import DelayedCall, ReactorBase, ThreadedResolver
 from twisted.internet.defer import Deferred
-from twisted.internet.base import ThreadedResolver, DelayedCall, ReactorBase
+from twisted.internet.error import DNSLookupError
+from twisted.internet.interfaces import IReactorThreads, IReactorTime, IResolverSimple
 from twisted.internet.task import Clock
-from twisted.trial.unittest import TestCase, SkipTest
+from twisted.python.threadpool import ThreadPool
+from twisted.trial.unittest import SkipTest, TestCase
+
+try:
+    import signal as _signal
+except ImportError:
+    signal = None
+else:
+    signal = _signal
+
+_P = ParamSpec("_P")
 
 
 @implementer(IReactorTime, IReactorThreads)
-class FakeReactor(object):
+class FakeReactor:
     """
     A fake reactor implementation which just supports enough reactor APIs for
     L{ThreadedResolver}.
@@ -41,25 +50,42 @@ class FakeReactor(object):
 
         self._threadCalls = Queue()
 
-
-    def callFromThread(self, f, *args, **kwargs):
-        self._threadCalls.put((f, args, kwargs))
-
+    def callFromThread(
+        self, callable: Callable[_P, object], *args: _P.args, **kwargs: _P.kwargs
+    ) -> None:
+        self._threadCalls.put((callable, args, kwargs))
 
     def _runThreadCalls(self):
         f, args, kwargs = self._threadCalls.get()
         f(*args, **kwargs)
 
-
     def _stop(self):
         self._threadpool.stop()
 
+    def getDelayedCalls(self):
+        # IReactorTime.getDelayedCalls
+        pass
+
+    def seconds(self) -> float:  # type: ignore[empty-body]
+        # IReactorTime.seconds
+        pass
+
+    def callInThread(
+        self, callable: Callable[_P, object], *args: _P.args, **kwargs: _P.kwargs
+    ) -> None:
+        # IReactorInThreads.callInThread
+        pass
+
+    def suggestThreadPoolSize(self, size):
+        # IReactorThreads.suggestThreadPoolSize
+        pass
 
 
 class ThreadedResolverTests(TestCase):
     """
     Tests for L{ThreadedResolver}.
     """
+
     def test_success(self):
         """
         L{ThreadedResolver.getHostByName} returns a L{Deferred} which fires
@@ -75,11 +101,12 @@ class ThreadedResolverTests(TestCase):
 
         lookedUp = []
         resolvedTo = []
+
         def fakeGetHostByName(name):
             lookedUp.append(name)
             return ip
 
-        self.patch(socket, 'gethostbyname', fakeGetHostByName)
+        self.patch(socket, "gethostbyname", fakeGetHostByName)
 
         resolver = ThreadedResolver(reactor)
         d = resolver.getHostByName(name, (timeout,))
@@ -94,7 +121,6 @@ class ThreadedResolverTests(TestCase):
         reactor._clock.advance(timeout + 1)
         self.assertEqual(reactor._clock.calls, [])
 
-
     def test_failure(self):
         """
         L{ThreadedResolver.getHostByName} returns a L{Deferred} which fires a
@@ -106,9 +132,9 @@ class ThreadedResolverTests(TestCase):
         self.addCleanup(reactor._stop)
 
         def fakeGetHostByName(name):
-            raise IOError("ENOBUFS (this is a funny joke)")
+            raise OSError("ENOBUFS (this is a funny joke)")
 
-        self.patch(socket, 'gethostbyname', fakeGetHostByName)
+        self.patch(socket, "gethostbyname", fakeGetHostByName)
 
         failedWith = []
         resolver = ThreadedResolver(reactor)
@@ -124,7 +150,6 @@ class ThreadedResolverTests(TestCase):
         reactor._clock.advance(timeout + 1)
         self.assertEqual(reactor._clock.calls, [])
 
-
     def test_timeout(self):
         """
         If L{socket.gethostbyname} does not complete before the specified
@@ -137,10 +162,11 @@ class ThreadedResolverTests(TestCase):
         self.addCleanup(reactor._stop)
 
         result = Queue()
+
         def fakeGetHostByName(name):
             raise result.get()
 
-        self.patch(socket, 'gethostbyname', fakeGetHostByName)
+        self.patch(socket, "gethostbyname", fakeGetHostByName)
 
         failedWith = []
         resolver = ThreadedResolver(reactor)
@@ -157,7 +183,6 @@ class ThreadedResolverTests(TestCase):
         # an exception.  Nobody cares, though.
         result.put(IOError("The I/O was errorful"))
 
-
     def test_resolverGivenStr(self):
         """
         L{ThreadedResolver.getHostByName} is passed L{str}, encoded using IDNA
@@ -166,7 +191,7 @@ class ThreadedResolverTests(TestCase):
         calls = []
 
         @implementer(IResolverSimple)
-        class FakeResolver(object):
+        class FakeResolver:
             def getHostByName(self, name, timeouts=()):
                 calls.append(name)
                 return Deferred()
@@ -179,19 +204,14 @@ class ThreadedResolverTests(TestCase):
         reactor = JustEnoughReactor()
         reactor.installResolver(fake)
         rec = FirstOneWins(Deferred())
-        reactor.nameResolver.resolveHostName(
-            rec, u"example.example")
-        reactor.nameResolver.resolveHostName(
-            rec, "example.example")
-        reactor.nameResolver.resolveHostName(
-            rec, u"v\xe4\xe4ntynyt.example")
-        reactor.nameResolver.resolveHostName(
-            rec, u"\u0440\u0444.example")
-        reactor.nameResolver.resolveHostName(
-            rec, "xn----7sbb4ac0ad0be6cf.xn--p1ai")
+        reactor.nameResolver.resolveHostName(rec, "example.example")
+        reactor.nameResolver.resolveHostName(rec, "example.example")
+        reactor.nameResolver.resolveHostName(rec, "v\xe4\xe4ntynyt.example")
+        reactor.nameResolver.resolveHostName(rec, "\u0440\u0444.example")
+        reactor.nameResolver.resolveHostName(rec, "xn----7sbb4ac0ad0be6cf.xn--p1ai")
 
         self.assertEqual(len(calls), 5)
-        self.assertEqual(list(map(type, calls)), [str]*5)
+        self.assertEqual(list(map(type, calls)), [str] * 5)
         self.assertEqual("example.example", calls[0])
         self.assertEqual("example.example", calls[1])
         self.assertEqual("xn--vntynyt-5waa.example", calls[2])
@@ -199,18 +219,17 @@ class ThreadedResolverTests(TestCase):
         self.assertEqual("xn----7sbb4ac0ad0be6cf.xn--p1ai", calls[4])
 
 
-
 def nothing():
     """
     Function used by L{DelayedCallTests.test_str}.
     """
 
 
-
-class DelayedCallMixin(object):
+class DelayedCallMixin:
     """
     L{DelayedCall}
     """
+
     def _getDelayedCallAt(self, time):
         """
         Get a L{DelayedCall} instance at a given C{time}.
@@ -218,10 +237,11 @@ class DelayedCallMixin(object):
         @param time: The absolute time at which the returned L{DelayedCall}
             will be scheduled.
         """
+
         def noop(call):
             pass
-        return DelayedCall(time, lambda: None, (), {}, noop, noop, None)
 
+        return DelayedCall(time, lambda: None, (), {}, noop, noop, None)
 
     def setUp(self):
         """
@@ -231,19 +251,26 @@ class DelayedCallMixin(object):
         self.zero = self._getDelayedCallAt(0)
         self.one = self._getDelayedCallAt(1)
 
-
     def test_str(self):
         """
         The string representation of a L{DelayedCall} instance, as returned by
         L{str}, includes the unsigned id of the instance, as well as its state,
         the function to be called, and the function arguments.
         """
-        dc = DelayedCall(12, nothing, (3, ), {"A": 5}, None, None, lambda: 1.5)
+        dc = DelayedCall(12, nothing, (3,), {"A": 5}, None, None, lambda: 1.5)
         self.assertEqual(
             str(dc),
             "<DelayedCall 0x%x [10.5s] called=0 cancelled=0 nothing(3, A=5)>"
-                % (id(dc),))
+            % (id(dc),),
+        )
 
+    def test_repr(self):
+        """
+        The string representation of a L{DelayedCall} instance, as returned by
+        {repr}, is identical to that returned by L{str}.
+        """
+        dc = DelayedCall(13, nothing, (6,), {"A": 9}, None, None, lambda: 1.6)
+        self.assertEqual(str(dc), repr(dc))
 
     def test_lt(self):
         """
@@ -256,7 +283,6 @@ class DelayedCallMixin(object):
         self.assertFalse(zero < zero)
         self.assertFalse(one < one)
 
-
     def test_le(self):
         """
         For two instances of L{DelayedCall} C{a} and C{b}, C{a <= b} is true
@@ -269,7 +295,6 @@ class DelayedCallMixin(object):
         self.assertTrue(zero <= zero)
         self.assertTrue(one <= one)
 
-
     def test_gt(self):
         """
         For two instances of L{DelayedCall} C{a} and C{b}, C{a > b} is true
@@ -281,7 +306,6 @@ class DelayedCallMixin(object):
         self.assertFalse(zero > zero)
         self.assertFalse(one > one)
 
-
     def test_ge(self):
         """
         For two instances of L{DelayedCall} C{a} and C{b}, C{a > b} is true
@@ -294,7 +318,6 @@ class DelayedCallMixin(object):
         self.assertTrue(zero >= zero)
         self.assertTrue(one >= one)
 
-
     def test_eq(self):
         """
         A L{DelayedCall} instance is only equal to itself.
@@ -305,7 +328,6 @@ class DelayedCallMixin(object):
         self.assertTrue(self.zero == self.zero)
         self.assertTrue(self.one == self.one)
 
-
     def test_ne(self):
         """
         A L{DelayedCall} instance is not equal to any other object.
@@ -317,59 +339,79 @@ class DelayedCallMixin(object):
         self.assertFalse(self.one != self.one)
 
 
-
 class DelayedCallNoDebugTests(DelayedCallMixin, TestCase):
     """
     L{DelayedCall}
     """
+
     def setUp(self):
         """
         Turn debug off.
         """
-        self.patch(DelayedCall, 'debug', False)
+        self.patch(DelayedCall, "debug", False)
         DelayedCallMixin.setUp(self)
 
-
     def test_str(self):
         """
         The string representation of a L{DelayedCall} instance, as returned by
         L{str}, includes the unsigned id of the instance, as well as its state,
         the function to be called, and the function arguments.
         """
-        dc = DelayedCall(12, nothing, (3, ), {"A": 5}, None, None, lambda: 1.5)
+        dc = DelayedCall(12, nothing, (3,), {"A": 5}, None, None, lambda: 1.5)
         expected = (
             "<DelayedCall 0x{:x} [10.5s] called=0 cancelled=0 "
-            "nothing(3, A=5)>".format(id(dc)))
+            "nothing(3, A=5)>".format(id(dc))
+        )
         self.assertEqual(str(dc), expected)
 
+    def test_switchToDebug(self):
+        """
+        If L{DelayedCall.debug} changes from C{0} to C{1} between
+        L{DelayeCall.__init__} and L{DelayedCall.__repr__} then
+        L{DelayedCall.__repr__} returns a string that does not include the
+        creator stack.
+        """
+        dc = DelayedCall(3, lambda: None, (), {}, nothing, nothing, lambda: 2)
+        dc.debug = 1
+        self.assertNotIn("traceback at creation", repr(dc))
 
 
 class DelayedCallDebugTests(DelayedCallMixin, TestCase):
     """
     L{DelayedCall}
     """
+
     def setUp(self):
         """
         Turn debug on.
         """
-        self.patch(DelayedCall, 'debug', True)
+        self.patch(DelayedCall, "debug", True)
         DelayedCallMixin.setUp(self)
 
-
     def test_str(self):
         """
         The string representation of a L{DelayedCall} instance, as returned by
         L{str}, includes the unsigned id of the instance, as well as its state,
         the function to be called, and the function arguments.
         """
-        dc = DelayedCall(12, nothing, (3, ), {"A": 5}, None, None, lambda: 1.5)
+        dc = DelayedCall(12, nothing, (3,), {"A": 5}, None, None, lambda: 1.5)
         expectedRegexp = (
             "<DelayedCall 0x{:x} \\[10.5s\\] called=0 cancelled=0 "
             "nothing\\(3, A=5\\)\n\n"
-            "traceback at creation:".format(id(dc)))
-        self.assertRegex(
-            str(dc), expectedRegexp)
+            "traceback at creation:".format(id(dc))
+        )
+        self.assertRegex(str(dc), expectedRegexp)
 
+    def test_switchFromDebug(self):
+        """
+        If L{DelayedCall.debug} changes from C{1} to C{0} between
+        L{DelayeCall.__init__} and L{DelayedCall.__repr__} then
+        L{DelayedCall.__repr__} returns a string that includes the creator
+        stack (we captured it, we might as well display it).
+        """
+        dc = DelayedCall(3, lambda: None, (), {}, nothing, nothing, lambda: 2)
+        dc.debug = 0
+        self.assertIn("traceback at creation", repr(dc))
 
 
 class TestSpySignalCapturingReactor(ReactorBase):
@@ -385,7 +427,7 @@ class TestSpySignalCapturingReactor(ReactorBase):
         """
 
 
-
+@skipIf(not signal, "signal module not available")
 class ReactorBaseSignalTests(TestCase):
 
     """
@@ -399,7 +441,6 @@ class ReactorBaseSignalTests(TestCase):
         reactor = TestSpySignalCapturingReactor()
         self.assertIs(None, reactor._exitSignal)
 
-
     def test_captureSIGINT(self):
         """
         ReactorBase's SIGINT handler saves the value of SIGINT to the
@@ -409,7 +450,6 @@ class ReactorBaseSignalTests(TestCase):
         reactor.sigInt(signal.SIGINT, None)
         self.assertEquals(signal.SIGINT, reactor._exitSignal)
 
-
     def test_captureSIGTERM(self):
         """
         ReactorBase's SIGTERM handler saves the value of SIGTERM to the
@@ -419,7 +459,6 @@ class ReactorBaseSignalTests(TestCase):
         reactor.sigTerm(signal.SIGTERM, None)
         self.assertEquals(signal.SIGTERM, reactor._exitSignal)
 
-
     def test_captureSIGBREAK(self):
         """
         ReactorBase's SIGBREAK handler saves the value of SIGBREAK to the
@@ -431,10 +470,3 @@ class ReactorBaseSignalTests(TestCase):
         reactor = TestSpySignalCapturingReactor()
         reactor.sigBreak(signal.SIGBREAK, None)
         self.assertEquals(signal.SIGBREAK, reactor._exitSignal)
-
-
-
-try:
-    import signal
-except ImportError:
-    ReactorBaseSignalTests.skip = "signal module not available"
